// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _SNAP7_CONNECTION_H_
#define _SNAP7_CONNECTION_H_

#include <silecs-communication/protocol/core/silecs.h>
#include <silecs-communication/interface/communication/SilecsConnection.h>
#include <snap7.h>

namespace Silecs
{
	class PLC;

	/*!
	 * \class SNAP7Connection
	 * \brief Plc-communication object for specific SNAP7 protocol
	 */
	class SNAP7Connection : public Connection
	{

	public:
		SNAP7Connection(PLC* thePLC);
 		virtual ~SNAP7Connection();

        int readUnitCode(PLC* thePLC, UnitCodeType& dataStruct);
        int readUnitStatus(PLC* thePLC, UnitStatusType& dataStruct);
        int readCPUInfo(PLC* thePLC, CPUInfoType& dataStruct);
        int readCPInfo(PLC* thePLC, CPInfoType& dataStruct);

        bool isRunning(PLC* thePLC);

 		int readData(PLC* thePLC, unsigned long DBn, unsigned long offset, unsigned long size, unsigned char* pBuffer);
 		int writeData(PLC* thePLC, unsigned long DBn, unsigned long offset, unsigned long size, unsigned char* pBuffer);

 		//Extension Silecs methods
 		int coldRestart(PLC* thePLC);
 		int plcStop(PLC* thePLC);

	private:
 		S7Object recvClient_;
        S7Object sendClient_;
		bool open(PLC* thePLC);
		bool close(PLC* thePLC);

        int rackNb_; //rackNb - common rack is 0 by default
        int slotNb_; //slotNb - depends on hardware configuration (scan is required the first connect)

	    std::string getSNAP7ErrorMessage(int err);
	};

} // namespace

#endif // _SNAP7_CONNECTION_H_

