// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <silecs-communication/interface/core/SilecsService.h>
#include <silecs-communication/interface/utility/Thread.h>
#include <silecs-communication/interface/utility/XMLParser.h>
#include <silecs-communication/interface/equipment/SilecsPLC.h>
#include <silecs-communication/interface/equipment/SilecsBlock.h>
#include <silecs-communication/interface/equipment/SilecsRegister.h>
#include <silecs-communication/interface/core/SilecsAction.h>
#include <silecs-communication/interface/core/PLCRecvAction.h>
#include <silecs-communication/interface/core/PLCSendAction.h>
#include <silecs-communication/interface/core/WrapperAction.h>
#include <silecs-communication/interface/utility/SilecsException.h>
#include <silecs-communication/interface/utility/SilecsLog.h>
#include <silecs-communication/interface/utility/StringUtilities.h>
#include <silecs-communication/interface/equipment/PLCBlock.h>

namespace Silecs
{

	PLCBlock::PLCBlock(PLC* thePLC,ElementXML blockNode, AccessType accessType):
			Block(thePLC, blockNode,accessType)
	{
		// Create buffer for the block exchanges
		bufferSize_ = memSize_; //size of one block type (including alignements)
		if (getPLC()->getProtocolID() == BlockMode)
		{ //BlockMode ==> access all devices in once
		  bufferSize_ *= getPLC()->getDeviceMap().size();
		}
		pBuffer_ = (unsigned char*)calloc(bufferSize_, sizeof(unsigned char));
	}

	PLCBlock::~PLCBlock()
	{
		//Remove the buffer
		if (pBuffer_ != NULL) free(pBuffer_); pBuffer_ = NULL;
	}

	InputBlock::InputBlock(PLC* thePLC,ElementXML blockNode, AccessType accessType) :
		PLCBlock(thePLC, blockNode, accessType)
	{
		if (DEBUG & Log::topics_) LOG(ALLOC) << "Block (create): " << name_ << ", plc: " << getPLC()->getName() << ", access: Input" << ", address: " << address_ << ", mem-size: " << memSize_ << ", buffer-size: " << bufferSize_;

        // Creates receive task which relies on the block exchange
		if (getPLC()->getProtocolID() == BlockMode)
			pAction_ = new PLCRecvBlockMode(this);
		else
			pAction_ = new PLCRecvDeviceMode(this);

		WrapperAction wrapperAction(pAction_);
		pTask_ = new Task<WrapperAction>(WrapperAction::function, wrapperAction);
	}


	InputBlock::~InputBlock()
	{
	}


	OutputBlock::OutputBlock(PLC* thePLC,ElementXML blockNode, AccessType accessType) :
		PLCBlock(thePLC, blockNode, accessType)
	{
		if (DEBUG & Log::topics_) LOG(ALLOC) << "Block (create): " << name_ << ", plc: " << getPLC()->getName() << ", access: Output" << ", address: " << address_ << ", mem-size: " << memSize_ << ", buffer-size: " << bufferSize_;

        // Creates send task which relies on the block exchange
		if (getPLC()->getProtocolID() == BlockMode)
			pAction_ = new PLCSendBlockMode(this);
		else
			pAction_ = new PLCSendDeviceMode(this);

		WrapperAction wrapperAction(pAction_);
		pTask_ = new Task<WrapperAction>(WrapperAction::function, wrapperAction);
	}

	OutputBlock::~OutputBlock()
	{
	}

}
